# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .summary import Summary
from ..._models import BaseModel
from .sessions.message import Message

__all__ = ["SessionContextResponse"]


class SessionContextResponse(BaseModel):
    id: str

    messages: List[Message]

    peer_card: Optional[List[str]] = None
    """The peer card, if context is requested from a specific perspective"""

    peer_representation: Optional[str] = None
    """
    A curated subset of a peer representation, if context is requested from a
    specific perspective
    """

    summary: Optional[Summary] = None
    """The summary if available"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal

import httpx

from .sessions import (
    SessionsResource,
    AsyncSessionsResource,
    SessionsResourceWithRawResponse,
    AsyncSessionsResourceWithRawResponse,
    SessionsResourceWithStreamingResponse,
    AsyncSessionsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncPage, AsyncPage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.workspaces import (
    peer_card_params,
    peer_chat_params,
    peer_list_params,
    peer_search_params,
    peer_update_params,
    peer_context_params,
    peer_set_card_params,
    peer_get_or_create_params,
    peer_representation_params,
)
from ....types.workspaces.peer import Peer
from ....types.workspaces.peer_card_response import PeerCardResponse
from ....types.workspaces.peer_chat_response import PeerChatResponse
from ....types.workspaces.peer_search_response import PeerSearchResponse
from ....types.workspaces.peer_context_response import PeerContextResponse
from ....types.workspaces.peer_representation_response import PeerRepresentationResponse

__all__ = ["PeersResource", "AsyncPeersResource"]


class PeersResource(SyncAPIResource):
    @cached_property
    def sessions(self) -> SessionsResource:
        return SessionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> PeersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/plastic-labs/honcho-python-core#accessing-raw-response-data-eg-headers
        """
        return PeersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PeersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/plastic-labs/honcho-python-core#with_streaming_response
        """
        return PeersResourceWithStreamingResponse(self)

    def update(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        configuration: Optional[Dict[str, object]] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Peer:
        """
        Update a Peer's metadata and/or configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return self._put(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}",
            body=maybe_transform(
                {
                    "configuration": configuration,
                    "metadata": metadata,
                },
                peer_update_params.PeerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Peer,
        )

    def list(
        self,
        workspace_id: str,
        *,
        page: int | Omit = omit,
        size: int | Omit = omit,
        filters: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPage[Peer]:
        """
        Get all Peers for a Workspace, paginated with optional filters.

        Args:
          page: Page number

          size: Page size

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return self._get_api_list(
            f"/v2/workspaces/{workspace_id}/peers/list",
            page=SyncPage[Peer],
            body=maybe_transform({"filters": filters}, peer_list_params.PeerListParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "size": size,
                    },
                    peer_list_params.PeerListParams,
                ),
            ),
            model=Peer,
            method="post",
        )

    def card(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        target: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PeerCardResponse:
        """
        Get a peer card for a specific peer relationship.

        Returns the peer card that the observer peer has for the target peer if it
        exists. If no target is specified, returns the observer's own peer card.

        Args:
          peer_id: ID of the observer peer

          target: Optional target peer to retrieve a card for, from the observer's perspective. If
              not provided, returns the observer's own card

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return self._get(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}/card",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"target": target}, peer_card_params.PeerCardParams),
            ),
            cast_to=PeerCardResponse,
        )

    def chat(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        query: str,
        reasoning_level: Literal["minimal", "low", "medium", "high", "max"] | Omit = omit,
        session_id: Optional[str] | Omit = omit,
        stream: bool | Omit = omit,
        target: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PeerChatResponse:
        """Query a Peer's representation using natural language.

        Performs agentic search
        and reasoning to comprehensively answer the query based on all latent knowledge
        gathered about the peer from their messages and conclusions.

        Args:
          query: Dialectic API Prompt

          reasoning_level: Level of reasoning to apply: minimal, low, medium, high, or max

          session_id: ID of the session to scope the representation to

          target: Optional peer to get the representation for, from the perspective of this peer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return self._post(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}/chat",
            body=maybe_transform(
                {
                    "query": query,
                    "reasoning_level": reasoning_level,
                    "session_id": session_id,
                    "stream": stream,
                    "target": target,
                },
                peer_chat_params.PeerChatParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PeerChatResponse,
        )

    def context(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        include_most_frequent: bool | Omit = omit,
        max_conclusions: Optional[int] | Omit = omit,
        search_max_distance: Optional[float] | Omit = omit,
        search_query: Optional[str] | Omit = omit,
        search_top_k: Optional[int] | Omit = omit,
        target: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PeerContextResponse:
        """
        Get context for a peer, including their representation and peer card.

        This endpoint returns a curated subset of the representation and peer card for a
        peer. If a target is specified, returns the context for the target from the
        observer peer's perspective. If no target is specified, returns the peer's own
        context (self-observation).

        This is useful for getting all the context needed about a peer without making
        multiple API calls.

        Args:
          peer_id: ID of the observer peer

          include_most_frequent: Whether to include the most frequent conclusions in the representation

          max_conclusions: Maximum number of conclusions to include in the representation

          search_max_distance: Only used if `search_query` is provided. Maximum distance for semantically
              relevant conclusions

          search_query: Optional query to curate the representation around semantic search results

          search_top_k: Only used if `search_query` is provided. Number of semantic-search-retrieved
              conclusions to include

          target: Optional target peer to get context for, from the observer's perspective. If not
              provided, returns the observer's own context (self-observation)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return self._get(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}/context",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_most_frequent": include_most_frequent,
                        "max_conclusions": max_conclusions,
                        "search_max_distance": search_max_distance,
                        "search_query": search_query,
                        "search_top_k": search_top_k,
                        "target": target,
                    },
                    peer_context_params.PeerContextParams,
                ),
            ),
            cast_to=PeerContextResponse,
        )

    def get_or_create(
        self,
        workspace_id: str,
        *,
        id: str,
        configuration: Optional[Dict[str, object]] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Peer:
        """
        Get a Peer by ID or create a new Peer with the given ID.

        If peer_id is provided as a query parameter, it uses that (must match JWT
        workspace_id). Otherwise, it uses the peer_id from the JWT.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return self._post(
            f"/v2/workspaces/{workspace_id}/peers",
            body=maybe_transform(
                {
                    "id": id,
                    "configuration": configuration,
                    "metadata": metadata,
                },
                peer_get_or_create_params.PeerGetOrCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Peer,
        )

    def representation(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        include_most_frequent: Optional[bool] | Omit = omit,
        max_conclusions: Optional[int] | Omit = omit,
        search_max_distance: Optional[float] | Omit = omit,
        search_query: Optional[str] | Omit = omit,
        search_top_k: Optional[int] | Omit = omit,
        session_id: Optional[str] | Omit = omit,
        target: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PeerRepresentationResponse:
        """Get a curated subset of a Peer's Representation.

        A Representation is always a
        subset of the total knowledge about the Peer. The subset can be scoped and
        filtered in various ways.

        If a session_id is provided in the body, we get the Representation of the Peer
        scoped to that Session. If a target is provided, we get the Representation of
        the target from the perspective of the Peer. If no target is provided, we get
        the omniscient Honcho Representation of the Peer.

        Args:
          include_most_frequent: Only used if `search_query` is provided. Whether to include the most frequent
              conclusions in the representation

          max_conclusions: Only used if `search_query` is provided. Maximum number of conclusions to
              include in the representation

          search_max_distance: Only used if `search_query` is provided. Maximum distance to search for
              semantically relevant conclusions

          search_query: Optional input to curate the representation around semantic search results

          search_top_k: Only used if `search_query` is provided. Number of semantic-search-retrieved
              conclusions to include in the representation

          session_id: Optional session ID within which to scope the representation

          target: Optional peer ID to get the representation for, from the perspective of this
              peer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return self._post(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}/representation",
            body=maybe_transform(
                {
                    "include_most_frequent": include_most_frequent,
                    "max_conclusions": max_conclusions,
                    "search_max_distance": search_max_distance,
                    "search_query": search_query,
                    "search_top_k": search_top_k,
                    "session_id": session_id,
                    "target": target,
                },
                peer_representation_params.PeerRepresentationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PeerRepresentationResponse,
        )

    def search(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        query: str,
        filters: Optional[Dict[str, object]] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PeerSearchResponse:
        """
        Search a Peer's messages, optionally filtered by various criteria.

        Args:
          query: Search query

          filters: Filters to scope the search

          limit: Number of results to return

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return self._post(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}/search",
            body=maybe_transform(
                {
                    "query": query,
                    "filters": filters,
                    "limit": limit,
                },
                peer_search_params.PeerSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PeerSearchResponse,
        )

    def set_card(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        peer_card: SequenceNotStr[str],
        target: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PeerCardResponse:
        """
        Set a peer card for a specific peer relationship.

        Sets the peer card that the observer peer has for the target peer. If no target
        is specified, sets the observer's own peer card.

        Args:
          peer_id: ID of the observer peer

          peer_card: The peer card content to set

          target: Optional target peer to set a card for, from the observer's perspective. If not
              provided, sets the observer's own card

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return self._put(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}/card",
            body=maybe_transform({"peer_card": peer_card}, peer_set_card_params.PeerSetCardParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"target": target}, peer_set_card_params.PeerSetCardParams),
            ),
            cast_to=PeerCardResponse,
        )


class AsyncPeersResource(AsyncAPIResource):
    @cached_property
    def sessions(self) -> AsyncSessionsResource:
        return AsyncSessionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPeersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/plastic-labs/honcho-python-core#accessing-raw-response-data-eg-headers
        """
        return AsyncPeersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPeersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/plastic-labs/honcho-python-core#with_streaming_response
        """
        return AsyncPeersResourceWithStreamingResponse(self)

    async def update(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        configuration: Optional[Dict[str, object]] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Peer:
        """
        Update a Peer's metadata and/or configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return await self._put(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}",
            body=await async_maybe_transform(
                {
                    "configuration": configuration,
                    "metadata": metadata,
                },
                peer_update_params.PeerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Peer,
        )

    def list(
        self,
        workspace_id: str,
        *,
        page: int | Omit = omit,
        size: int | Omit = omit,
        filters: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Peer, AsyncPage[Peer]]:
        """
        Get all Peers for a Workspace, paginated with optional filters.

        Args:
          page: Page number

          size: Page size

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return self._get_api_list(
            f"/v2/workspaces/{workspace_id}/peers/list",
            page=AsyncPage[Peer],
            body=maybe_transform({"filters": filters}, peer_list_params.PeerListParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "size": size,
                    },
                    peer_list_params.PeerListParams,
                ),
            ),
            model=Peer,
            method="post",
        )

    async def card(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        target: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PeerCardResponse:
        """
        Get a peer card for a specific peer relationship.

        Returns the peer card that the observer peer has for the target peer if it
        exists. If no target is specified, returns the observer's own peer card.

        Args:
          peer_id: ID of the observer peer

          target: Optional target peer to retrieve a card for, from the observer's perspective. If
              not provided, returns the observer's own card

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return await self._get(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}/card",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"target": target}, peer_card_params.PeerCardParams),
            ),
            cast_to=PeerCardResponse,
        )

    async def chat(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        query: str,
        reasoning_level: Literal["minimal", "low", "medium", "high", "max"] | Omit = omit,
        session_id: Optional[str] | Omit = omit,
        stream: bool | Omit = omit,
        target: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PeerChatResponse:
        """Query a Peer's representation using natural language.

        Performs agentic search
        and reasoning to comprehensively answer the query based on all latent knowledge
        gathered about the peer from their messages and conclusions.

        Args:
          query: Dialectic API Prompt

          reasoning_level: Level of reasoning to apply: minimal, low, medium, high, or max

          session_id: ID of the session to scope the representation to

          target: Optional peer to get the representation for, from the perspective of this peer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return await self._post(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}/chat",
            body=await async_maybe_transform(
                {
                    "query": query,
                    "reasoning_level": reasoning_level,
                    "session_id": session_id,
                    "stream": stream,
                    "target": target,
                },
                peer_chat_params.PeerChatParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PeerChatResponse,
        )

    async def context(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        include_most_frequent: bool | Omit = omit,
        max_conclusions: Optional[int] | Omit = omit,
        search_max_distance: Optional[float] | Omit = omit,
        search_query: Optional[str] | Omit = omit,
        search_top_k: Optional[int] | Omit = omit,
        target: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PeerContextResponse:
        """
        Get context for a peer, including their representation and peer card.

        This endpoint returns a curated subset of the representation and peer card for a
        peer. If a target is specified, returns the context for the target from the
        observer peer's perspective. If no target is specified, returns the peer's own
        context (self-observation).

        This is useful for getting all the context needed about a peer without making
        multiple API calls.

        Args:
          peer_id: ID of the observer peer

          include_most_frequent: Whether to include the most frequent conclusions in the representation

          max_conclusions: Maximum number of conclusions to include in the representation

          search_max_distance: Only used if `search_query` is provided. Maximum distance for semantically
              relevant conclusions

          search_query: Optional query to curate the representation around semantic search results

          search_top_k: Only used if `search_query` is provided. Number of semantic-search-retrieved
              conclusions to include

          target: Optional target peer to get context for, from the observer's perspective. If not
              provided, returns the observer's own context (self-observation)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return await self._get(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}/context",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include_most_frequent": include_most_frequent,
                        "max_conclusions": max_conclusions,
                        "search_max_distance": search_max_distance,
                        "search_query": search_query,
                        "search_top_k": search_top_k,
                        "target": target,
                    },
                    peer_context_params.PeerContextParams,
                ),
            ),
            cast_to=PeerContextResponse,
        )

    async def get_or_create(
        self,
        workspace_id: str,
        *,
        id: str,
        configuration: Optional[Dict[str, object]] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Peer:
        """
        Get a Peer by ID or create a new Peer with the given ID.

        If peer_id is provided as a query parameter, it uses that (must match JWT
        workspace_id). Otherwise, it uses the peer_id from the JWT.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return await self._post(
            f"/v2/workspaces/{workspace_id}/peers",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "configuration": configuration,
                    "metadata": metadata,
                },
                peer_get_or_create_params.PeerGetOrCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Peer,
        )

    async def representation(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        include_most_frequent: Optional[bool] | Omit = omit,
        max_conclusions: Optional[int] | Omit = omit,
        search_max_distance: Optional[float] | Omit = omit,
        search_query: Optional[str] | Omit = omit,
        search_top_k: Optional[int] | Omit = omit,
        session_id: Optional[str] | Omit = omit,
        target: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PeerRepresentationResponse:
        """Get a curated subset of a Peer's Representation.

        A Representation is always a
        subset of the total knowledge about the Peer. The subset can be scoped and
        filtered in various ways.

        If a session_id is provided in the body, we get the Representation of the Peer
        scoped to that Session. If a target is provided, we get the Representation of
        the target from the perspective of the Peer. If no target is provided, we get
        the omniscient Honcho Representation of the Peer.

        Args:
          include_most_frequent: Only used if `search_query` is provided. Whether to include the most frequent
              conclusions in the representation

          max_conclusions: Only used if `search_query` is provided. Maximum number of conclusions to
              include in the representation

          search_max_distance: Only used if `search_query` is provided. Maximum distance to search for
              semantically relevant conclusions

          search_query: Optional input to curate the representation around semantic search results

          search_top_k: Only used if `search_query` is provided. Number of semantic-search-retrieved
              conclusions to include in the representation

          session_id: Optional session ID within which to scope the representation

          target: Optional peer ID to get the representation for, from the perspective of this
              peer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return await self._post(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}/representation",
            body=await async_maybe_transform(
                {
                    "include_most_frequent": include_most_frequent,
                    "max_conclusions": max_conclusions,
                    "search_max_distance": search_max_distance,
                    "search_query": search_query,
                    "search_top_k": search_top_k,
                    "session_id": session_id,
                    "target": target,
                },
                peer_representation_params.PeerRepresentationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PeerRepresentationResponse,
        )

    async def search(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        query: str,
        filters: Optional[Dict[str, object]] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PeerSearchResponse:
        """
        Search a Peer's messages, optionally filtered by various criteria.

        Args:
          query: Search query

          filters: Filters to scope the search

          limit: Number of results to return

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return await self._post(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}/search",
            body=await async_maybe_transform(
                {
                    "query": query,
                    "filters": filters,
                    "limit": limit,
                },
                peer_search_params.PeerSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PeerSearchResponse,
        )

    async def set_card(
        self,
        peer_id: str,
        *,
        workspace_id: str,
        peer_card: SequenceNotStr[str],
        target: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PeerCardResponse:
        """
        Set a peer card for a specific peer relationship.

        Sets the peer card that the observer peer has for the target peer. If no target
        is specified, sets the observer's own peer card.

        Args:
          peer_id: ID of the observer peer

          peer_card: The peer card content to set

          target: Optional target peer to set a card for, from the observer's perspective. If not
              provided, sets the observer's own card

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not peer_id:
            raise ValueError(f"Expected a non-empty value for `peer_id` but received {peer_id!r}")
        return await self._put(
            f"/v2/workspaces/{workspace_id}/peers/{peer_id}/card",
            body=await async_maybe_transform({"peer_card": peer_card}, peer_set_card_params.PeerSetCardParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"target": target}, peer_set_card_params.PeerSetCardParams),
            ),
            cast_to=PeerCardResponse,
        )


class PeersResourceWithRawResponse:
    def __init__(self, peers: PeersResource) -> None:
        self._peers = peers

        self.update = to_raw_response_wrapper(
            peers.update,
        )
        self.list = to_raw_response_wrapper(
            peers.list,
        )
        self.card = to_raw_response_wrapper(
            peers.card,
        )
        self.chat = to_raw_response_wrapper(
            peers.chat,
        )
        self.context = to_raw_response_wrapper(
            peers.context,
        )
        self.get_or_create = to_raw_response_wrapper(
            peers.get_or_create,
        )
        self.representation = to_raw_response_wrapper(
            peers.representation,
        )
        self.search = to_raw_response_wrapper(
            peers.search,
        )
        self.set_card = to_raw_response_wrapper(
            peers.set_card,
        )

    @cached_property
    def sessions(self) -> SessionsResourceWithRawResponse:
        return SessionsResourceWithRawResponse(self._peers.sessions)


class AsyncPeersResourceWithRawResponse:
    def __init__(self, peers: AsyncPeersResource) -> None:
        self._peers = peers

        self.update = async_to_raw_response_wrapper(
            peers.update,
        )
        self.list = async_to_raw_response_wrapper(
            peers.list,
        )
        self.card = async_to_raw_response_wrapper(
            peers.card,
        )
        self.chat = async_to_raw_response_wrapper(
            peers.chat,
        )
        self.context = async_to_raw_response_wrapper(
            peers.context,
        )
        self.get_or_create = async_to_raw_response_wrapper(
            peers.get_or_create,
        )
        self.representation = async_to_raw_response_wrapper(
            peers.representation,
        )
        self.search = async_to_raw_response_wrapper(
            peers.search,
        )
        self.set_card = async_to_raw_response_wrapper(
            peers.set_card,
        )

    @cached_property
    def sessions(self) -> AsyncSessionsResourceWithRawResponse:
        return AsyncSessionsResourceWithRawResponse(self._peers.sessions)


class PeersResourceWithStreamingResponse:
    def __init__(self, peers: PeersResource) -> None:
        self._peers = peers

        self.update = to_streamed_response_wrapper(
            peers.update,
        )
        self.list = to_streamed_response_wrapper(
            peers.list,
        )
        self.card = to_streamed_response_wrapper(
            peers.card,
        )
        self.chat = to_streamed_response_wrapper(
            peers.chat,
        )
        self.context = to_streamed_response_wrapper(
            peers.context,
        )
        self.get_or_create = to_streamed_response_wrapper(
            peers.get_or_create,
        )
        self.representation = to_streamed_response_wrapper(
            peers.representation,
        )
        self.search = to_streamed_response_wrapper(
            peers.search,
        )
        self.set_card = to_streamed_response_wrapper(
            peers.set_card,
        )

    @cached_property
    def sessions(self) -> SessionsResourceWithStreamingResponse:
        return SessionsResourceWithStreamingResponse(self._peers.sessions)


class AsyncPeersResourceWithStreamingResponse:
    def __init__(self, peers: AsyncPeersResource) -> None:
        self._peers = peers

        self.update = async_to_streamed_response_wrapper(
            peers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            peers.list,
        )
        self.card = async_to_streamed_response_wrapper(
            peers.card,
        )
        self.chat = async_to_streamed_response_wrapper(
            peers.chat,
        )
        self.context = async_to_streamed_response_wrapper(
            peers.context,
        )
        self.get_or_create = async_to_streamed_response_wrapper(
            peers.get_or_create,
        )
        self.representation = async_to_streamed_response_wrapper(
            peers.representation,
        )
        self.search = async_to_streamed_response_wrapper(
            peers.search,
        )
        self.set_card = async_to_streamed_response_wrapper(
            peers.set_card,
        )

    @cached_property
    def sessions(self) -> AsyncSessionsResourceWithStreamingResponse:
        return AsyncSessionsResourceWithStreamingResponse(self._peers.sessions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.workspaces import queue_status_params
from ...types.workspaces.queue_status_response import QueueStatusResponse

__all__ = ["QueueResource", "AsyncQueueResource"]


class QueueResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> QueueResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/plastic-labs/honcho-python-core#accessing-raw-response-data-eg-headers
        """
        return QueueResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QueueResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/plastic-labs/honcho-python-core#with_streaming_response
        """
        return QueueResourceWithStreamingResponse(self)

    def status(
        self,
        workspace_id: str,
        *,
        observer_id: Optional[str] | Omit = omit,
        sender_id: Optional[str] | Omit = omit,
        session_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueStatusResponse:
        """
        Get the processing queue status for a Workspace, optionally scoped to an
        observer, sender, and/or session.

        Args:
          observer_id: Optional observer ID to filter by

          sender_id: Optional sender ID to filter by

          session_id: Optional session ID to filter by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return self._get(
            f"/v2/workspaces/{workspace_id}/queue/status",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "observer_id": observer_id,
                        "sender_id": sender_id,
                        "session_id": session_id,
                    },
                    queue_status_params.QueueStatusParams,
                ),
            ),
            cast_to=QueueStatusResponse,
        )


class AsyncQueueResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncQueueResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/plastic-labs/honcho-python-core#accessing-raw-response-data-eg-headers
        """
        return AsyncQueueResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQueueResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/plastic-labs/honcho-python-core#with_streaming_response
        """
        return AsyncQueueResourceWithStreamingResponse(self)

    async def status(
        self,
        workspace_id: str,
        *,
        observer_id: Optional[str] | Omit = omit,
        sender_id: Optional[str] | Omit = omit,
        session_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueStatusResponse:
        """
        Get the processing queue status for a Workspace, optionally scoped to an
        observer, sender, and/or session.

        Args:
          observer_id: Optional observer ID to filter by

          sender_id: Optional sender ID to filter by

          session_id: Optional session ID to filter by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return await self._get(
            f"/v2/workspaces/{workspace_id}/queue/status",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "observer_id": observer_id,
                        "sender_id": sender_id,
                        "session_id": session_id,
                    },
                    queue_status_params.QueueStatusParams,
                ),
            ),
            cast_to=QueueStatusResponse,
        )


class QueueResourceWithRawResponse:
    def __init__(self, queue: QueueResource) -> None:
        self._queue = queue

        self.status = to_raw_response_wrapper(
            queue.status,
        )


class AsyncQueueResourceWithRawResponse:
    def __init__(self, queue: AsyncQueueResource) -> None:
        self._queue = queue

        self.status = async_to_raw_response_wrapper(
            queue.status,
        )


class QueueResourceWithStreamingResponse:
    def __init__(self, queue: QueueResource) -> None:
        self._queue = queue

        self.status = to_streamed_response_wrapper(
            queue.status,
        )


class AsyncQueueResourceWithStreamingResponse:
    def __init__(self, queue: AsyncQueueResource) -> None:
        self._queue = queue

        self.status = async_to_streamed_response_wrapper(
            queue.status,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from honcho_core import Honcho, AsyncHoncho
from tests.utils import assert_matches_type
from honcho_core.pagination import SyncPage, AsyncPage
from honcho_core.types.workspaces import (
    Session,
    SessionSearchResponse,
    SessionContextResponse,
    SessionSummariesResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSessions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_update(self, client: Honcho) -> None:
        session = client.workspaces.sessions.update(
            session_id="session_id",
            workspace_id="workspace_id",
        )
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Honcho) -> None:
        session = client.workspaces.sessions.update(
            session_id="session_id",
            workspace_id="workspace_id",
            configuration={
                "dream": {"enabled": True},
                "peer_card": {
                    "create": True,
                    "use": True,
                },
                "reasoning": {
                    "custom_instructions": "custom_instructions",
                    "enabled": True,
                },
                "summary": {
                    "enabled": True,
                    "messages_per_long_summary": 20,
                    "messages_per_short_summary": 10,
                },
            },
            metadata={"foo": "bar"},
        )
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Honcho) -> None:
        response = client.workspaces.sessions.with_raw_response.update(
            session_id="session_id",
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Honcho) -> None:
        with client.workspaces.sessions.with_streaming_response.update(
            session_id="session_id",
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(Session, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Honcho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            client.workspaces.sessions.with_raw_response.update(
                session_id="session_id",
                workspace_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.workspaces.sessions.with_raw_response.update(
                session_id="",
                workspace_id="workspace_id",
            )

    @parametrize
    def test_method_list(self, client: Honcho) -> None:
        session = client.workspaces.sessions.list(
            workspace_id="workspace_id",
        )
        assert_matches_type(SyncPage[Session], session, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Honcho) -> None:
        session = client.workspaces.sessions.list(
            workspace_id="workspace_id",
            page=1,
            size=1,
            filters={"foo": "bar"},
        )
        assert_matches_type(SyncPage[Session], session, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Honcho) -> None:
        response = client.workspaces.sessions.with_raw_response.list(
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SyncPage[Session], session, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Honcho) -> None:
        with client.workspaces.sessions.with_streaming_response.list(
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SyncPage[Session], session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: Honcho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            client.workspaces.sessions.with_raw_response.list(
                workspace_id="",
            )

    @parametrize
    def test_method_delete(self, client: Honcho) -> None:
        session = client.workspaces.sessions.delete(
            session_id="session_id",
            workspace_id="workspace_id",
        )
        assert_matches_type(object, session, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Honcho) -> None:
        response = client.workspaces.sessions.with_raw_response.delete(
            session_id="session_id",
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(object, session, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Honcho) -> None:
        with client.workspaces.sessions.with_streaming_response.delete(
            session_id="session_id",
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(object, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Honcho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            client.workspaces.sessions.with_raw_response.delete(
                session_id="session_id",
                workspace_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.workspaces.sessions.with_raw_response.delete(
                session_id="",
                workspace_id="workspace_id",
            )

    @parametrize
    def test_method_clone(self, client: Honcho) -> None:
        session = client.workspaces.sessions.clone(
            session_id="session_id",
            workspace_id="workspace_id",
        )
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    def test_method_clone_with_all_params(self, client: Honcho) -> None:
        session = client.workspaces.sessions.clone(
            session_id="session_id",
            workspace_id="workspace_id",
            message_id="message_id",
        )
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    def test_raw_response_clone(self, client: Honcho) -> None:
        response = client.workspaces.sessions.with_raw_response.clone(
            session_id="session_id",
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    def test_streaming_response_clone(self, client: Honcho) -> None:
        with client.workspaces.sessions.with_streaming_response.clone(
            session_id="session_id",
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(Session, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_clone(self, client: Honcho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            client.workspaces.sessions.with_raw_response.clone(
                session_id="session_id",
                workspace_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.workspaces.sessions.with_raw_response.clone(
                session_id="",
                workspace_id="workspace_id",
            )

    @parametrize
    def test_method_context(self, client: Honcho) -> None:
        session = client.workspaces.sessions.context(
            session_id="session_id",
            workspace_id="workspace_id",
        )
        assert_matches_type(SessionContextResponse, session, path=["response"])

    @parametrize
    def test_method_context_with_all_params(self, client: Honcho) -> None:
        session = client.workspaces.sessions.context(
            session_id="session_id",
            workspace_id="workspace_id",
            include_most_frequent=True,
            last_message="last_message",
            limit_to_session=True,
            max_conclusions=1,
            peer_perspective="peer_perspective",
            peer_target="peer_target",
            search_max_distance=0,
            search_top_k=1,
            summary=True,
            tokens=100000,
        )
        assert_matches_type(SessionContextResponse, session, path=["response"])

    @parametrize
    def test_raw_response_context(self, client: Honcho) -> None:
        response = client.workspaces.sessions.with_raw_response.context(
            session_id="session_id",
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SessionContextResponse, session, path=["response"])

    @parametrize
    def test_streaming_response_context(self, client: Honcho) -> None:
        with client.workspaces.sessions.with_streaming_response.context(
            session_id="session_id",
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SessionContextResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_context(self, client: Honcho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            client.workspaces.sessions.with_raw_response.context(
                session_id="session_id",
                workspace_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.workspaces.sessions.with_raw_response.context(
                session_id="",
                workspace_id="workspace_id",
            )

    @parametrize
    def test_method_get_or_create(self, client: Honcho) -> None:
        session = client.workspaces.sessions.get_or_create(
            workspace_id="workspace_id",
            id="id",
        )
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    def test_method_get_or_create_with_all_params(self, client: Honcho) -> None:
        session = client.workspaces.sessions.get_or_create(
            workspace_id="workspace_id",
            id="id",
            configuration={
                "dream": {"enabled": True},
                "peer_card": {
                    "create": True,
                    "use": True,
                },
                "reasoning": {
                    "custom_instructions": "custom_instructions",
                    "enabled": True,
                },
                "summary": {
                    "enabled": True,
                    "messages_per_long_summary": 20,
                    "messages_per_short_summary": 10,
                },
            },
            metadata={"foo": "bar"},
            peers={
                "foo": {
                    "observe_me": True,
                    "observe_others": True,
                }
            },
        )
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    def test_raw_response_get_or_create(self, client: Honcho) -> None:
        response = client.workspaces.sessions.with_raw_response.get_or_create(
            workspace_id="workspace_id",
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    def test_streaming_response_get_or_create(self, client: Honcho) -> None:
        with client.workspaces.sessions.with_streaming_response.get_or_create(
            workspace_id="workspace_id",
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(Session, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_get_or_create(self, client: Honcho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            client.workspaces.sessions.with_raw_response.get_or_create(
                workspace_id="",
                id="id",
            )

    @parametrize
    def test_method_search(self, client: Honcho) -> None:
        session = client.workspaces.sessions.search(
            session_id="session_id",
            workspace_id="workspace_id",
            query="query",
        )
        assert_matches_type(SessionSearchResponse, session, path=["response"])

    @parametrize
    def test_method_search_with_all_params(self, client: Honcho) -> None:
        session = client.workspaces.sessions.search(
            session_id="session_id",
            workspace_id="workspace_id",
            query="query",
            filters={"foo": "bar"},
            limit=1,
        )
        assert_matches_type(SessionSearchResponse, session, path=["response"])

    @parametrize
    def test_raw_response_search(self, client: Honcho) -> None:
        response = client.workspaces.sessions.with_raw_response.search(
            session_id="session_id",
            workspace_id="workspace_id",
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SessionSearchResponse, session, path=["response"])

    @parametrize
    def test_streaming_response_search(self, client: Honcho) -> None:
        with client.workspaces.sessions.with_streaming_response.search(
            session_id="session_id",
            workspace_id="workspace_id",
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SessionSearchResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_search(self, client: Honcho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            client.workspaces.sessions.with_raw_response.search(
                session_id="session_id",
                workspace_id="",
                query="query",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.workspaces.sessions.with_raw_response.search(
                session_id="",
                workspace_id="workspace_id",
                query="query",
            )

    @parametrize
    def test_method_summaries(self, client: Honcho) -> None:
        session = client.workspaces.sessions.summaries(
            session_id="session_id",
            workspace_id="workspace_id",
        )
        assert_matches_type(SessionSummariesResponse, session, path=["response"])

    @parametrize
    def test_raw_response_summaries(self, client: Honcho) -> None:
        response = client.workspaces.sessions.with_raw_response.summaries(
            session_id="session_id",
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SessionSummariesResponse, session, path=["response"])

    @parametrize
    def test_streaming_response_summaries(self, client: Honcho) -> None:
        with client.workspaces.sessions.with_streaming_response.summaries(
            session_id="session_id",
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SessionSummariesResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_summaries(self, client: Honcho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            client.workspaces.sessions.with_raw_response.summaries(
                session_id="session_id",
                workspace_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.workspaces.sessions.with_raw_response.summaries(
                session_id="",
                workspace_id="workspace_id",
            )


class TestAsyncSessions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_update(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.update(
            session_id="session_id",
            workspace_id="workspace_id",
        )
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.update(
            session_id="session_id",
            workspace_id="workspace_id",
            configuration={
                "dream": {"enabled": True},
                "peer_card": {
                    "create": True,
                    "use": True,
                },
                "reasoning": {
                    "custom_instructions": "custom_instructions",
                    "enabled": True,
                },
                "summary": {
                    "enabled": True,
                    "messages_per_long_summary": 20,
                    "messages_per_short_summary": 10,
                },
            },
            metadata={"foo": "bar"},
        )
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHoncho) -> None:
        response = await async_client.workspaces.sessions.with_raw_response.update(
            session_id="session_id",
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHoncho) -> None:
        async with async_client.workspaces.sessions.with_streaming_response.update(
            session_id="session_id",
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(Session, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncHoncho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.update(
                session_id="session_id",
                workspace_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.update(
                session_id="",
                workspace_id="workspace_id",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.list(
            workspace_id="workspace_id",
        )
        assert_matches_type(AsyncPage[Session], session, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.list(
            workspace_id="workspace_id",
            page=1,
            size=1,
            filters={"foo": "bar"},
        )
        assert_matches_type(AsyncPage[Session], session, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHoncho) -> None:
        response = await async_client.workspaces.sessions.with_raw_response.list(
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(AsyncPage[Session], session, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHoncho) -> None:
        async with async_client.workspaces.sessions.with_streaming_response.list(
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(AsyncPage[Session], session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncHoncho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.list(
                workspace_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.delete(
            session_id="session_id",
            workspace_id="workspace_id",
        )
        assert_matches_type(object, session, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncHoncho) -> None:
        response = await async_client.workspaces.sessions.with_raw_response.delete(
            session_id="session_id",
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(object, session, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncHoncho) -> None:
        async with async_client.workspaces.sessions.with_streaming_response.delete(
            session_id="session_id",
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(object, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncHoncho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.delete(
                session_id="session_id",
                workspace_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.delete(
                session_id="",
                workspace_id="workspace_id",
            )

    @parametrize
    async def test_method_clone(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.clone(
            session_id="session_id",
            workspace_id="workspace_id",
        )
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    async def test_method_clone_with_all_params(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.clone(
            session_id="session_id",
            workspace_id="workspace_id",
            message_id="message_id",
        )
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    async def test_raw_response_clone(self, async_client: AsyncHoncho) -> None:
        response = await async_client.workspaces.sessions.with_raw_response.clone(
            session_id="session_id",
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    async def test_streaming_response_clone(self, async_client: AsyncHoncho) -> None:
        async with async_client.workspaces.sessions.with_streaming_response.clone(
            session_id="session_id",
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(Session, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_clone(self, async_client: AsyncHoncho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.clone(
                session_id="session_id",
                workspace_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.clone(
                session_id="",
                workspace_id="workspace_id",
            )

    @parametrize
    async def test_method_context(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.context(
            session_id="session_id",
            workspace_id="workspace_id",
        )
        assert_matches_type(SessionContextResponse, session, path=["response"])

    @parametrize
    async def test_method_context_with_all_params(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.context(
            session_id="session_id",
            workspace_id="workspace_id",
            include_most_frequent=True,
            last_message="last_message",
            limit_to_session=True,
            max_conclusions=1,
            peer_perspective="peer_perspective",
            peer_target="peer_target",
            search_max_distance=0,
            search_top_k=1,
            summary=True,
            tokens=100000,
        )
        assert_matches_type(SessionContextResponse, session, path=["response"])

    @parametrize
    async def test_raw_response_context(self, async_client: AsyncHoncho) -> None:
        response = await async_client.workspaces.sessions.with_raw_response.context(
            session_id="session_id",
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(SessionContextResponse, session, path=["response"])

    @parametrize
    async def test_streaming_response_context(self, async_client: AsyncHoncho) -> None:
        async with async_client.workspaces.sessions.with_streaming_response.context(
            session_id="session_id",
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(SessionContextResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_context(self, async_client: AsyncHoncho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.context(
                session_id="session_id",
                workspace_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.context(
                session_id="",
                workspace_id="workspace_id",
            )

    @parametrize
    async def test_method_get_or_create(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.get_or_create(
            workspace_id="workspace_id",
            id="id",
        )
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    async def test_method_get_or_create_with_all_params(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.get_or_create(
            workspace_id="workspace_id",
            id="id",
            configuration={
                "dream": {"enabled": True},
                "peer_card": {
                    "create": True,
                    "use": True,
                },
                "reasoning": {
                    "custom_instructions": "custom_instructions",
                    "enabled": True,
                },
                "summary": {
                    "enabled": True,
                    "messages_per_long_summary": 20,
                    "messages_per_short_summary": 10,
                },
            },
            metadata={"foo": "bar"},
            peers={
                "foo": {
                    "observe_me": True,
                    "observe_others": True,
                }
            },
        )
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    async def test_raw_response_get_or_create(self, async_client: AsyncHoncho) -> None:
        response = await async_client.workspaces.sessions.with_raw_response.get_or_create(
            workspace_id="workspace_id",
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(Session, session, path=["response"])

    @parametrize
    async def test_streaming_response_get_or_create(self, async_client: AsyncHoncho) -> None:
        async with async_client.workspaces.sessions.with_streaming_response.get_or_create(
            workspace_id="workspace_id",
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(Session, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_get_or_create(self, async_client: AsyncHoncho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.get_or_create(
                workspace_id="",
                id="id",
            )

    @parametrize
    async def test_method_search(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.search(
            session_id="session_id",
            workspace_id="workspace_id",
            query="query",
        )
        assert_matches_type(SessionSearchResponse, session, path=["response"])

    @parametrize
    async def test_method_search_with_all_params(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.search(
            session_id="session_id",
            workspace_id="workspace_id",
            query="query",
            filters={"foo": "bar"},
            limit=1,
        )
        assert_matches_type(SessionSearchResponse, session, path=["response"])

    @parametrize
    async def test_raw_response_search(self, async_client: AsyncHoncho) -> None:
        response = await async_client.workspaces.sessions.with_raw_response.search(
            session_id="session_id",
            workspace_id="workspace_id",
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(SessionSearchResponse, session, path=["response"])

    @parametrize
    async def test_streaming_response_search(self, async_client: AsyncHoncho) -> None:
        async with async_client.workspaces.sessions.with_streaming_response.search(
            session_id="session_id",
            workspace_id="workspace_id",
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(SessionSearchResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_search(self, async_client: AsyncHoncho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.search(
                session_id="session_id",
                workspace_id="",
                query="query",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.search(
                session_id="",
                workspace_id="workspace_id",
                query="query",
            )

    @parametrize
    async def test_method_summaries(self, async_client: AsyncHoncho) -> None:
        session = await async_client.workspaces.sessions.summaries(
            session_id="session_id",
            workspace_id="workspace_id",
        )
        assert_matches_type(SessionSummariesResponse, session, path=["response"])

    @parametrize
    async def test_raw_response_summaries(self, async_client: AsyncHoncho) -> None:
        response = await async_client.workspaces.sessions.with_raw_response.summaries(
            session_id="session_id",
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(SessionSummariesResponse, session, path=["response"])

    @parametrize
    async def test_streaming_response_summaries(self, async_client: AsyncHoncho) -> None:
        async with async_client.workspaces.sessions.with_streaming_response.summaries(
            session_id="session_id",
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(SessionSummariesResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_summaries(self, async_client: AsyncHoncho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.summaries(
                session_id="session_id",
                workspace_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.workspaces.sessions.with_raw_response.summaries(
                session_id="",
                workspace_id="workspace_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["PeerContextParams"]


class PeerContextParams(TypedDict, total=False):
    workspace_id: Required[str]

    include_most_frequent: bool
    """Whether to include the most frequent conclusions in the representation"""

    max_conclusions: Optional[int]
    """Maximum number of conclusions to include in the representation"""

    search_max_distance: Optional[float]
    """Only used if `search_query` is provided.

    Maximum distance for semantically relevant conclusions
    """

    search_query: Optional[str]
    """Optional query to curate the representation around semantic search results"""

    search_top_k: Optional[int]
    """Only used if `search_query` is provided.

    Number of semantic-search-retrieved conclusions to include
    """

    target: Optional[str]
    """Optional target peer to get context for, from the observer's perspective.

    If not provided, returns the observer's own context (self-observation)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["PeerRepresentationParams"]


class PeerRepresentationParams(TypedDict, total=False):
    workspace_id: Required[str]

    include_most_frequent: Optional[bool]
    """Only used if `search_query` is provided.

    Whether to include the most frequent conclusions in the representation
    """

    max_conclusions: Optional[int]
    """Only used if `search_query` is provided.

    Maximum number of conclusions to include in the representation
    """

    search_max_distance: Optional[float]
    """Only used if `search_query` is provided.

    Maximum distance to search for semantically relevant conclusions
    """

    search_query: Optional[str]
    """Optional input to curate the representation around semantic search results"""

    search_top_k: Optional[int]
    """Only used if `search_query` is provided.

    Number of semantic-search-retrieved conclusions to include in the representation
    """

    session_id: Optional[str]
    """Optional session ID within which to scope the representation"""

    target: Optional[str]
    """
    Optional peer ID to get the representation for, from the perspective of this
    peer
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional

import httpx

from .peers import (
    PeersResource,
    AsyncPeersResource,
    PeersResourceWithRawResponse,
    AsyncPeersResourceWithRawResponse,
    PeersResourceWithStreamingResponse,
    AsyncPeersResourceWithStreamingResponse,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncPage, AsyncPage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.workspaces import (
    session_list_params,
    session_clone_params,
    session_search_params,
    session_update_params,
    session_context_params,
    session_get_or_create_params,
)
from ....types.workspaces.session import Session
from ....types.workspaces.session_search_response import SessionSearchResponse
from ....types.workspaces.session_context_response import SessionContextResponse
from ....types.workspaces.session_summaries_response import SessionSummariesResponse
from ....types.workspaces.session_configuration_param import SessionConfigurationParam
from ....types.workspaces.sessions.session_peer_config_param import SessionPeerConfigParam

__all__ = ["SessionsResource", "AsyncSessionsResource"]


class SessionsResource(SyncAPIResource):
    @cached_property
    def messages(self) -> MessagesResource:
        return MessagesResource(self._client)

    @cached_property
    def peers(self) -> PeersResource:
        return PeersResource(self._client)

    @cached_property
    def with_raw_response(self) -> SessionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/plastic-labs/honcho-python-core#accessing-raw-response-data-eg-headers
        """
        return SessionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SessionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/plastic-labs/honcho-python-core#with_streaming_response
        """
        return SessionsResourceWithStreamingResponse(self)

    def update(
        self,
        session_id: str,
        *,
        workspace_id: str,
        configuration: Optional[SessionConfigurationParam] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Session:
        """
        Update a Session's metadata and/or configuration.

        Args:
          configuration: The set of options that can be in a session DB-level configuration dictionary.

              All fields are optional. Session-level configuration overrides workspace-level
              configuration, which overrides global configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._put(
            f"/v2/workspaces/{workspace_id}/sessions/{session_id}",
            body=maybe_transform(
                {
                    "configuration": configuration,
                    "metadata": metadata,
                },
                session_update_params.SessionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Session,
        )

    def list(
        self,
        workspace_id: str,
        *,
        page: int | Omit = omit,
        size: int | Omit = omit,
        filters: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPage[Session]:
        """
        Get all Sessions for a Workspace, paginated with optional filters.

        Args:
          page: Page number

          size: Page size

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return self._get_api_list(
            f"/v2/workspaces/{workspace_id}/sessions/list",
            page=SyncPage[Session],
            body=maybe_transform({"filters": filters}, session_list_params.SessionListParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "size": size,
                    },
                    session_list_params.SessionListParams,
                ),
            ),
            model=Session,
            method="post",
        )

    def delete(
        self,
        session_id: str,
        *,
        workspace_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a Session and all associated messages.

        The Session is marked as inactive immediately and returns 202 Accepted. The
        actual deletion of all related data happens asynchronously via the queue with
        retry support.

        This action cannot be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._delete(
            f"/v2/workspaces/{workspace_id}/sessions/{session_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def clone(
        self,
        session_id: str,
        *,
        workspace_id: str,
        message_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Session:
        """
        Clone a Session, optionally up to a specific message ID.

        Args:
          message_id: Message ID to cut off the clone at

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._post(
            f"/v2/workspaces/{workspace_id}/sessions/{session_id}/clone",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"message_id": message_id}, session_clone_params.SessionCloneParams),
            ),
            cast_to=Session,
        )

    def context(
        self,
        session_id: str,
        *,
        workspace_id: str,
        include_most_frequent: bool | Omit = omit,
        last_message: Optional[str] | Omit = omit,
        limit_to_session: bool | Omit = omit,
        max_conclusions: Optional[int] | Omit = omit,
        peer_perspective: Optional[str] | Omit = omit,
        peer_target: Optional[str] | Omit = omit,
        search_max_distance: Optional[float] | Omit = omit,
        search_top_k: Optional[int] | Omit = omit,
        summary: bool | Omit = omit,
        tokens: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionContextResponse:
        """Produce a context object from the Session.

        The caller provides an optional token
        limit which the entire context must fit into. If not provided, the context will
        be exhaustive (within configured max tokens). To do this, we allocate 40% of the
        token limit to the summary, and 60% to recent messages -- as many as can fit.
        Note that the summary will usually take up less space than this. If the caller
        does not want a summary, we allocate all the tokens to recent messages.

        Args:
          include_most_frequent: Only used if `last_message` is provided. Whether to include the most frequent
              conclusions in the representation

          last_message: The most recent message, used to fetch semantically relevant conclusions

          limit_to_session: Only used if `last_message` is provided. Whether to limit the representation to
              the session (as opposed to everything known about the target peer)

          max_conclusions: Only used if `last_message` is provided. The maximum number of conclusions to
              include in the representation

          peer_perspective: A peer to get context for. If given, response will attempt to include
              representation and card from the perspective of that peer. Must be provided with
              `peer_target`.

          peer_target: The target of the perspective. If given without `peer_perspective`, will get the
              Honcho-level representation and peer card for this peer. If given with
              `peer_perspective`, will get the representation and card for this peer _from the
              perspective of that peer_.

          search_max_distance: Only used if `last_message` is provided. The maximum distance to search for
              semantically relevant conclusions

          search_top_k: Only used if `last_message` is provided. The number of semantic-search-retrieved
              conclusions to include in the representation

          summary: Whether or not to include a summary _if_ one is available for the session

          tokens: Number of tokens to use for the context. Includes summary if set to true.
              Includes representation and peer card if they are included in the response. If
              not provided, the context will be exhaustive (within 100000 tokens)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._get(
            f"/v2/workspaces/{workspace_id}/sessions/{session_id}/context",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_most_frequent": include_most_frequent,
                        "last_message": last_message,
                        "limit_to_session": limit_to_session,
                        "max_conclusions": max_conclusions,
                        "peer_perspective": peer_perspective,
                        "peer_target": peer_target,
                        "search_max_distance": search_max_distance,
                        "search_top_k": search_top_k,
                        "summary": summary,
                        "tokens": tokens,
                    },
                    session_context_params.SessionContextParams,
                ),
            ),
            cast_to=SessionContextResponse,
        )

    def get_or_create(
        self,
        workspace_id: str,
        *,
        id: str,
        configuration: Optional[SessionConfigurationParam] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        peers: Optional[Dict[str, SessionPeerConfigParam]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Session:
        """
        Get a Session by ID or create a new Session with the given ID.

        If Session ID is provided as a parameter, it verifies the Session is in the
        Workspace. Otherwise, it uses the session_id from the JWT for verification.

        Args:
          configuration: The set of options that can be in a session DB-level configuration dictionary.

              All fields are optional. Session-level configuration overrides workspace-level
              configuration, which overrides global configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return self._post(
            f"/v2/workspaces/{workspace_id}/sessions",
            body=maybe_transform(
                {
                    "id": id,
                    "configuration": configuration,
                    "metadata": metadata,
                    "peers": peers,
                },
                session_get_or_create_params.SessionGetOrCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Session,
        )

    def search(
        self,
        session_id: str,
        *,
        workspace_id: str,
        query: str,
        filters: Optional[Dict[str, object]] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionSearchResponse:
        """Search a Session with optional filters.

        Use `limit` to control the number of
        results returned.

        Args:
          query: Search query

          filters: Filters to scope the search

          limit: Number of results to return

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._post(
            f"/v2/workspaces/{workspace_id}/sessions/{session_id}/search",
            body=maybe_transform(
                {
                    "query": query,
                    "filters": filters,
                    "limit": limit,
                },
                session_search_params.SessionSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionSearchResponse,
        )

    def summaries(
        self,
        session_id: str,
        *,
        workspace_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionSummariesResponse:
        """
        Get available summaries for a Session.

        Returns both short and long summaries if available, including metadata like the
        message ID they cover up to, creation timestamp, and token count.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._get(
            f"/v2/workspaces/{workspace_id}/sessions/{session_id}/summaries",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionSummariesResponse,
        )


class AsyncSessionsResource(AsyncAPIResource):
    @cached_property
    def messages(self) -> AsyncMessagesResource:
        return AsyncMessagesResource(self._client)

    @cached_property
    def peers(self) -> AsyncPeersResource:
        return AsyncPeersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSessionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/plastic-labs/honcho-python-core#accessing-raw-response-data-eg-headers
        """
        return AsyncSessionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSessionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/plastic-labs/honcho-python-core#with_streaming_response
        """
        return AsyncSessionsResourceWithStreamingResponse(self)

    async def update(
        self,
        session_id: str,
        *,
        workspace_id: str,
        configuration: Optional[SessionConfigurationParam] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Session:
        """
        Update a Session's metadata and/or configuration.

        Args:
          configuration: The set of options that can be in a session DB-level configuration dictionary.

              All fields are optional. Session-level configuration overrides workspace-level
              configuration, which overrides global configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._put(
            f"/v2/workspaces/{workspace_id}/sessions/{session_id}",
            body=await async_maybe_transform(
                {
                    "configuration": configuration,
                    "metadata": metadata,
                },
                session_update_params.SessionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Session,
        )

    def list(
        self,
        workspace_id: str,
        *,
        page: int | Omit = omit,
        size: int | Omit = omit,
        filters: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Session, AsyncPage[Session]]:
        """
        Get all Sessions for a Workspace, paginated with optional filters.

        Args:
          page: Page number

          size: Page size

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return self._get_api_list(
            f"/v2/workspaces/{workspace_id}/sessions/list",
            page=AsyncPage[Session],
            body=maybe_transform({"filters": filters}, session_list_params.SessionListParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "size": size,
                    },
                    session_list_params.SessionListParams,
                ),
            ),
            model=Session,
            method="post",
        )

    async def delete(
        self,
        session_id: str,
        *,
        workspace_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a Session and all associated messages.

        The Session is marked as inactive immediately and returns 202 Accepted. The
        actual deletion of all related data happens asynchronously via the queue with
        retry support.

        This action cannot be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._delete(
            f"/v2/workspaces/{workspace_id}/sessions/{session_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def clone(
        self,
        session_id: str,
        *,
        workspace_id: str,
        message_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Session:
        """
        Clone a Session, optionally up to a specific message ID.

        Args:
          message_id: Message ID to cut off the clone at

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._post(
            f"/v2/workspaces/{workspace_id}/sessions/{session_id}/clone",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"message_id": message_id}, session_clone_params.SessionCloneParams),
            ),
            cast_to=Session,
        )

    async def context(
        self,
        session_id: str,
        *,
        workspace_id: str,
        include_most_frequent: bool | Omit = omit,
        last_message: Optional[str] | Omit = omit,
        limit_to_session: bool | Omit = omit,
        max_conclusions: Optional[int] | Omit = omit,
        peer_perspective: Optional[str] | Omit = omit,
        peer_target: Optional[str] | Omit = omit,
        search_max_distance: Optional[float] | Omit = omit,
        search_top_k: Optional[int] | Omit = omit,
        summary: bool | Omit = omit,
        tokens: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionContextResponse:
        """Produce a context object from the Session.

        The caller provides an optional token
        limit which the entire context must fit into. If not provided, the context will
        be exhaustive (within configured max tokens). To do this, we allocate 40% of the
        token limit to the summary, and 60% to recent messages -- as many as can fit.
        Note that the summary will usually take up less space than this. If the caller
        does not want a summary, we allocate all the tokens to recent messages.

        Args:
          include_most_frequent: Only used if `last_message` is provided. Whether to include the most frequent
              conclusions in the representation

          last_message: The most recent message, used to fetch semantically relevant conclusions

          limit_to_session: Only used if `last_message` is provided. Whether to limit the representation to
              the session (as opposed to everything known about the target peer)

          max_conclusions: Only used if `last_message` is provided. The maximum number of conclusions to
              include in the representation

          peer_perspective: A peer to get context for. If given, response will attempt to include
              representation and card from the perspective of that peer. Must be provided with
              `peer_target`.

          peer_target: The target of the perspective. If given without `peer_perspective`, will get the
              Honcho-level representation and peer card for this peer. If given with
              `peer_perspective`, will get the representation and card for this peer _from the
              perspective of that peer_.

          search_max_distance: Only used if `last_message` is provided. The maximum distance to search for
              semantically relevant conclusions

          search_top_k: Only used if `last_message` is provided. The number of semantic-search-retrieved
              conclusions to include in the representation

          summary: Whether or not to include a summary _if_ one is available for the session

          tokens: Number of tokens to use for the context. Includes summary if set to true.
              Includes representation and peer card if they are included in the response. If
              not provided, the context will be exhaustive (within 100000 tokens)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._get(
            f"/v2/workspaces/{workspace_id}/sessions/{session_id}/context",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include_most_frequent": include_most_frequent,
                        "last_message": last_message,
                        "limit_to_session": limit_to_session,
                        "max_conclusions": max_conclusions,
                        "peer_perspective": peer_perspective,
                        "peer_target": peer_target,
                        "search_max_distance": search_max_distance,
                        "search_top_k": search_top_k,
                        "summary": summary,
                        "tokens": tokens,
                    },
                    session_context_params.SessionContextParams,
                ),
            ),
            cast_to=SessionContextResponse,
        )

    async def get_or_create(
        self,
        workspace_id: str,
        *,
        id: str,
        configuration: Optional[SessionConfigurationParam] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        peers: Optional[Dict[str, SessionPeerConfigParam]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Session:
        """
        Get a Session by ID or create a new Session with the given ID.

        If Session ID is provided as a parameter, it verifies the Session is in the
        Workspace. Otherwise, it uses the session_id from the JWT for verification.

        Args:
          configuration: The set of options that can be in a session DB-level configuration dictionary.

              All fields are optional. Session-level configuration overrides workspace-level
              configuration, which overrides global configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return await self._post(
            f"/v2/workspaces/{workspace_id}/sessions",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "configuration": configuration,
                    "metadata": metadata,
                    "peers": peers,
                },
                session_get_or_create_params.SessionGetOrCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Session,
        )

    async def search(
        self,
        session_id: str,
        *,
        workspace_id: str,
        query: str,
        filters: Optional[Dict[str, object]] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionSearchResponse:
        """Search a Session with optional filters.

        Use `limit` to control the number of
        results returned.

        Args:
          query: Search query

          filters: Filters to scope the search

          limit: Number of results to return

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._post(
            f"/v2/workspaces/{workspace_id}/sessions/{session_id}/search",
            body=await async_maybe_transform(
                {
                    "query": query,
                    "filters": filters,
                    "limit": limit,
                },
                session_search_params.SessionSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionSearchResponse,
        )

    async def summaries(
        self,
        session_id: str,
        *,
        workspace_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionSummariesResponse:
        """
        Get available summaries for a Session.

        Returns both short and long summaries if available, including metadata like the
        message ID they cover up to, creation timestamp, and token count.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._get(
            f"/v2/workspaces/{workspace_id}/sessions/{session_id}/summaries",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionSummariesResponse,
        )


class SessionsResourceWithRawResponse:
    def __init__(self, sessions: SessionsResource) -> None:
        self._sessions = sessions

        self.update = to_raw_response_wrapper(
            sessions.update,
        )
        self.list = to_raw_response_wrapper(
            sessions.list,
        )
        self.delete = to_raw_response_wrapper(
            sessions.delete,
        )
        self.clone = to_raw_response_wrapper(
            sessions.clone,
        )
        self.context = to_raw_response_wrapper(
            sessions.context,
        )
        self.get_or_create = to_raw_response_wrapper(
            sessions.get_or_create,
        )
        self.search = to_raw_response_wrapper(
            sessions.search,
        )
        self.summaries = to_raw_response_wrapper(
            sessions.summaries,
        )

    @cached_property
    def messages(self) -> MessagesResourceWithRawResponse:
        return MessagesResourceWithRawResponse(self._sessions.messages)

    @cached_property
    def peers(self) -> PeersResourceWithRawResponse:
        return PeersResourceWithRawResponse(self._sessions.peers)


class AsyncSessionsResourceWithRawResponse:
    def __init__(self, sessions: AsyncSessionsResource) -> None:
        self._sessions = sessions

        self.update = async_to_raw_response_wrapper(
            sessions.update,
        )
        self.list = async_to_raw_response_wrapper(
            sessions.list,
        )
        self.delete = async_to_raw_response_wrapper(
            sessions.delete,
        )
        self.clone = async_to_raw_response_wrapper(
            sessions.clone,
        )
        self.context = async_to_raw_response_wrapper(
            sessions.context,
        )
        self.get_or_create = async_to_raw_response_wrapper(
            sessions.get_or_create,
        )
        self.search = async_to_raw_response_wrapper(
            sessions.search,
        )
        self.summaries = async_to_raw_response_wrapper(
            sessions.summaries,
        )

    @cached_property
    def messages(self) -> AsyncMessagesResourceWithRawResponse:
        return AsyncMessagesResourceWithRawResponse(self._sessions.messages)

    @cached_property
    def peers(self) -> AsyncPeersResourceWithRawResponse:
        return AsyncPeersResourceWithRawResponse(self._sessions.peers)


class SessionsResourceWithStreamingResponse:
    def __init__(self, sessions: SessionsResource) -> None:
        self._sessions = sessions

        self.update = to_streamed_response_wrapper(
            sessions.update,
        )
        self.list = to_streamed_response_wrapper(
            sessions.list,
        )
        self.delete = to_streamed_response_wrapper(
            sessions.delete,
        )
        self.clone = to_streamed_response_wrapper(
            sessions.clone,
        )
        self.context = to_streamed_response_wrapper(
            sessions.context,
        )
        self.get_or_create = to_streamed_response_wrapper(
            sessions.get_or_create,
        )
        self.search = to_streamed_response_wrapper(
            sessions.search,
        )
        self.summaries = to_streamed_response_wrapper(
            sessions.summaries,
        )

    @cached_property
    def messages(self) -> MessagesResourceWithStreamingResponse:
        return MessagesResourceWithStreamingResponse(self._sessions.messages)

    @cached_property
    def peers(self) -> PeersResourceWithStreamingResponse:
        return PeersResourceWithStreamingResponse(self._sessions.peers)


class AsyncSessionsResourceWithStreamingResponse:
    def __init__(self, sessions: AsyncSessionsResource) -> None:
        self._sessions = sessions

        self.update = async_to_streamed_response_wrapper(
            sessions.update,
        )
        self.list = async_to_streamed_response_wrapper(
            sessions.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            sessions.delete,
        )
        self.clone = async_to_streamed_response_wrapper(
            sessions.clone,
        )
        self.context = async_to_streamed_response_wrapper(
            sessions.context,
        )
        self.get_or_create = async_to_streamed_response_wrapper(
            sessions.get_or_create,
        )
        self.search = async_to_streamed_response_wrapper(
            sessions.search,
        )
        self.summaries = async_to_streamed_response_wrapper(
            sessions.summaries,
        )

    @cached_property
    def messages(self) -> AsyncMessagesResourceWithStreamingResponse:
        return AsyncMessagesResourceWithStreamingResponse(self._sessions.messages)

    @cached_property
    def peers(self) -> AsyncPeersResourceWithStreamingResponse:
        return AsyncPeersResourceWithStreamingResponse(self._sessions.peers)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["PeerContextResponse"]


class PeerContextResponse(BaseModel):
    """Context for a peer, including representation and peer card."""

    peer_id: str
    """The ID of the peer"""

    target_id: str
    """The ID of the target peer being observed"""

    peer_card: Optional[List[str]] = None
    """The peer card for the target peer from the observer's perspective"""

    representation: Optional[str] = None
    """
    A curated subset of the representation of the target peer from the observer's
    perspective
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from honcho_core import Honcho, AsyncHoncho
from tests.utils import assert_matches_type
from honcho_core.types.workspaces import QueueStatusResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestQueue:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_status(self, client: Honcho) -> None:
        queue = client.workspaces.queue.status(
            workspace_id="workspace_id",
        )
        assert_matches_type(QueueStatusResponse, queue, path=["response"])

    @parametrize
    def test_method_status_with_all_params(self, client: Honcho) -> None:
        queue = client.workspaces.queue.status(
            workspace_id="workspace_id",
            observer_id="observer_id",
            sender_id="sender_id",
            session_id="session_id",
        )
        assert_matches_type(QueueStatusResponse, queue, path=["response"])

    @parametrize
    def test_raw_response_status(self, client: Honcho) -> None:
        response = client.workspaces.queue.with_raw_response.status(
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        queue = response.parse()
        assert_matches_type(QueueStatusResponse, queue, path=["response"])

    @parametrize
    def test_streaming_response_status(self, client: Honcho) -> None:
        with client.workspaces.queue.with_streaming_response.status(
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            queue = response.parse()
            assert_matches_type(QueueStatusResponse, queue, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_status(self, client: Honcho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            client.workspaces.queue.with_raw_response.status(
                workspace_id="",
            )


class TestAsyncQueue:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_status(self, async_client: AsyncHoncho) -> None:
        queue = await async_client.workspaces.queue.status(
            workspace_id="workspace_id",
        )
        assert_matches_type(QueueStatusResponse, queue, path=["response"])

    @parametrize
    async def test_method_status_with_all_params(self, async_client: AsyncHoncho) -> None:
        queue = await async_client.workspaces.queue.status(
            workspace_id="workspace_id",
            observer_id="observer_id",
            sender_id="sender_id",
            session_id="session_id",
        )
        assert_matches_type(QueueStatusResponse, queue, path=["response"])

    @parametrize
    async def test_raw_response_status(self, async_client: AsyncHoncho) -> None:
        response = await async_client.workspaces.queue.with_raw_response.status(
            workspace_id="workspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        queue = await response.parse()
        assert_matches_type(QueueStatusResponse, queue, path=["response"])

    @parametrize
    async def test_streaming_response_status(self, async_client: AsyncHoncho) -> None:
        async with async_client.workspaces.queue.with_streaming_response.status(
            workspace_id="workspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            queue = await response.parse()
            assert_matches_type(QueueStatusResponse, queue, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_status(self, async_client: AsyncHoncho) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `workspace_id` but received ''"):
            await async_client.workspaces.queue.with_raw_response.status(
                workspace_id="",
            )
